==> grafana-10.4.15.tar.gz: grafana-10.4.15/packages/grafana-data/src/field/fieldState.test.ts <==
import { toDataFrame } from '../dataframe';
import { DataFrame, TIME_SERIES_VALUE_FIELD_NAME, FieldType, TIME_SERIES_TIME_FIELD_NAME } from '../types';

import { getFieldDisplayName, getFrameDisplayName } from './fieldState';

interface TitleScenario {
  frames: DataFrame[];
  frameIndex?: number; // assume 0
  fieldIndex?: number; // assume 0
}

function checkScenario(scenario: TitleScenario): string {
  const frame = scenario.frames[scenario.frameIndex ?? 0];
  const field = frame.fields[scenario.fieldIndex ?? 0];
  return getFieldDisplayName(field, frame, scenario.frames);
}

describe('getFieldDisplayName', () => {
  it('Should add suffix for comparison frames', () => {
    const frame = toDataFrame({
      meta: {
        timeCompare: {
          diffMs: -86400000,
          isTimeShiftQuery: true,
        },
      },
      fields: [
        { name: TIME_SERIES_TIME_FIELD_NAME, values: [1, 2, 3], type: FieldType.time },
        {
          name: 'Value 1',
          values: [1, 2, 3],
          type: FieldType.number,
          config: {
            displayName: 'ServerA',
          },
        },
        {
          name: 'Value 2',
          values: [1, 2, 3],
          type: FieldType.number,
          config: {
            displayNameFromDS: 'ServerB',
          },
        },
        {
          name: 'Value 3',
          values: [1, 2, 3],
          type: FieldType.number,
        },
      ],
    });

    expect(getFieldDisplayName(frame.fields[1], frame)).toBe('ServerA (comparison)');
    expect(getFieldDisplayName(frame.fields[2], frame)).toBe('ServerB (comparison)');
    expect(getFieldDisplayName(frame.fields[3], frame)).toBe('Value 3 (comparison)');
  });
});

describe('getFrameDisplayName', () => {
  it('Should return frame name if set', () => {
    const frame = toDataFrame({
      name: 'Series A',
      fields: [{ name: 'Field 1' }],
    });
    expect(getFrameDisplayName(frame)).toBe('Series A');
  });

  it('Should return field name', () => {
    const frame = toDataFrame({
      fields: [{ name: 'Field 1' }],
    });
    expect(getFrameDisplayName(frame)).toBe('Field 1');
  });

  it('Should return all field names', () => {
    const frame = toDataFrame({
      fields: [{ name: 'Field A' }, { name: 'Field B' }],
    });
    expect(getFrameDisplayName(frame)).toBe('Field A, Field B');
  });

  it('Should return labels if single field with labels', () => {
    const frame = toDataFrame({
      fields: [{ name: 'value', labels: { server: 'A' } }],
    });

    expect(getFrameDisplayName(frame)).toBe('value A');
  });

  it('Should return field names when labels object exist but has no keys', () => {
    const frame = toDataFrame({
      fields: [{ name: 'value', labels: {} }],
    });
    expect(getFrameDisplayName(frame)).toBe('value');
  });

  it('Should return value field name if single value field', () => {
    const frame = toDataFrame({
      fields: [
        { name: TIME_SERIES_TIME_FIELD_NAME, values: [1, 2, 3], type: FieldType.time },
        {
          name: TIME_SERIES_VALUE_FIELD_NAME,
          values: [1, 2, 3],
          type: FieldType.number,
          config: {
            displayName: 'ServerA',
          },
        },
      ],
    });

    expect(getFrameDisplayName(frame, 1)).toBe('ServerA');
  });
});

describe('Check field state calculations (displayName and id)', () => {
  it('should use field name if no frame name', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          fields: [{ name: 'Field 1' }],
        }),
      ],
    });
    expect(title).toEqual('Field 1');
  });

  it('should use only field name if only one series', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          name: 'Series A',
          fields: [{ name: 'Field 1' }],
        }),
      ],
    });
    expect(title).toEqual('Field 1');
  });

  it('should use frame name and field name if more than one frame', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          name: 'Series A',
          fields: [{ name: 'Field 1' }],
        }),
        toDataFrame({
          name: 'Series B',
          fields: [{ name: 'Field 1' }],
        }),
      ],
    });
    expect(title).toEqual('Series A Field 1');
  });

  it('should add field name count to name if it exists more than once and is equal to TIME_SERIES_VALUE_FIELD_NAME', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          fields: [{ name: TIME_SERIES_VALUE_FIELD_NAME }, { name: TIME_SERIES_VALUE_FIELD_NAME }],
        }),
      ],
    });
    const title2 = checkScenario({
      frames: [
        toDataFrame({
          fields: [{ name: TIME_SERIES_VALUE_FIELD_NAME }, { name: TIME_SERIES_VALUE_FIELD_NAME }],
        }),
      ],
      fieldIndex: 1,
    });

    expect(title).toEqual('Value 1');
    expect(title2).toEqual('Value 2');
  });

  it('should add field name count to name if field name exists more than once', () => {
    const title2 = checkScenario({
      frames: [
        toDataFrame({
          fields: [{ name: 'A' }, { name: 'A' }],
        }),
      ],
      fieldIndex: 1,
    });

    expect(title2).toEqual('A 2');
  });

  it('should only use label value if only one label', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          fields: [{ name: 'Value', labels: { server: 'Server A' } }],
        }),
      ],
    });
    expect(title).toEqual('Server A');
  });

  it('should use label value only if all series have same name', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          name: 'cpu',
          fields: [{ name: 'Value', labels: { server: 'Server A' } }],
        }),
        toDataFrame({
          name: 'cpu',
          fields: [{ name: 'Value', labels: { server: 'Server A' } }],
        }),
      ],
    });
    expect(title).toEqual('Server A');
  });

  it('should use label name and value if more than one label', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          fields: [{ name: 'Value', labels: { server: 'Server A', mode: 'B' } }],
        }),
      ],
    });
    expect(title).toEqual('{mode="B", server="Server A"}');
  });

  it('should use field name even when it is TIME_SERIES_VALUE_FIELD_NAME if there are no labels', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          fields: [{ name: TIME_SERIES_VALUE_FIELD_NAME, labels: {} }],
        }),
      ],
    });
    expect(title).toEqual('Value');
  });

  it('should use series name when field name is TIME_SERIES_VALUE_FIELD_NAME and there are no labels ', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          name: 'Series A',
          fields: [{ name: TIME_SERIES_VALUE_FIELD_NAME, labels: {} }],
        }),
      ],
    });
    expect(title).toEqual('Series A');
  });

  it('should reder loki frames', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          refId: 'A',
          fields: [
            { name: 'time', type: FieldType.time },
            {
              name: 'line',
              labels: { host: 'ec2-13-53-116-156.eu-north-1.compute.amazonaws.com', region: 'eu-north1' },
            },
          ],
        }),
      ],
      fieldIndex: 1,
    });
    expect(title).toEqual('line {host="ec2-13-53-116-156.eu-north-1.compute.amazonaws.com", region="eu-north1"}');
  });
});

==> grafana-12.2.0.tar.gz: grafana-12.2.0/packages/grafana-data/src/field/fieldState.test.ts <==
import { toDataFrame } from '../dataframe/processDataFrame';
import { DataFrame, TIME_SERIES_TIME_FIELD_NAME, FieldType, TIME_SERIES_VALUE_FIELD_NAME } from '../types/dataFrame';

import { getFieldDisplayName, getFrameDisplayName } from './fieldState';

interface TitleScenario {
  frames: DataFrame[];
  frameIndex?: number; // assume 0
  fieldIndex?: number; // assume 0
}

function checkScenario(scenario: TitleScenario): string {
  const frame = scenario.frames[scenario.frameIndex ?? 0];
  const field = frame.fields[scenario.fieldIndex ?? 0];
  return getFieldDisplayName(field, frame, scenario.frames);
}

describe('getFieldDisplayName', () => {
  it('Should add suffix for comparison frames', () => {
    const frame = toDataFrame({
      meta: {
        timeCompare: {
          diffMs: -86400000,
          isTimeShiftQuery: true,
        },
      },
      fields: [
        { name: TIME_SERIES_TIME_FIELD_NAME, values: [1, 2, 3], type: FieldType.time },
        {
          name: 'Value 1',
          values: [1, 2, 3],
          type: FieldType.number,
          config: {
            displayName: 'ServerA',
          },
        },
        {
          name: 'Value 2',
          values: [1, 2, 3],
          type: FieldType.number,
          config: {
            displayNameFromDS: 'ServerB',
          },
        },
        {
          name: 'Value 3',
          values: [1, 2, 3],
          type: FieldType.number,
        },
      ],
    });

    expect(getFieldDisplayName(frame.fields[1], frame)).toBe('ServerA (comparison)');
    expect(getFieldDisplayName(frame.fields[2], frame)).toBe('ServerB (comparison)');
    expect(getFieldDisplayName(frame.fields[3], frame)).toBe('Value 3 (comparison)');
  });
});

describe('getFrameDisplayName', () => {
  it('Should return frame name if set', () => {
    const frame = toDataFrame({
      name: 'Series A',
      fields: [{ name: 'Field 1' }],
    });
    expect(getFrameDisplayName(frame)).toBe('Series A');
  });

  it('Should return field name', () => {
    const frame = toDataFrame({
      fields: [{ name: 'Field 1' }],
    });
    expect(getFrameDisplayName(frame)).toBe('Field 1');
  });

  it('Should return all field names', () => {
    const frame = toDataFrame({
      fields: [{ name: 'Field A' }, { name: 'Field B' }],
    });
    expect(getFrameDisplayName(frame)).toBe('Field A, Field B');
  });

  it('Should return labels if single field with labels', () => {
    const frame = toDataFrame({
      fields: [{ name: 'value', labels: { server: 'A' } }],
    });

    expect(getFrameDisplayName(frame)).toBe('value A');
  });

  it('Should return field names when labels object exist but has no keys', () => {
    const frame = toDataFrame({
      fields: [{ name: 'value', labels: {} }],
    });
    expect(getFrameDisplayName(frame)).toBe('value');
  });

  it('Should return value field name if single value field', () => {
    const frame = toDataFrame({
      fields: [
        { name: TIME_SERIES_TIME_FIELD_NAME, values: [1, 2, 3], type: FieldType.time },
        {
          name: TIME_SERIES_VALUE_FIELD_NAME,
          values: [1, 2, 3],
          type: FieldType.number,
          config: {
            displayName: 'ServerA',
          },
        },
      ],
    });

    expect(getFrameDisplayName(frame, 1)).toBe('ServerA');
  });
});

describe('Check field state calculations (displayName and id)', () => {
  it('should use field name if no frame name', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          fields: [{ name: 'Field 1' }],
        }),
      ],
    });
    expect(title).toEqual('Field 1');
  });

  it('should use only field name if only one series', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          name: 'Series A',
          fields: [{ name: 'Field 1' }],
        }),
      ],
    });
    expect(title).toEqual('Field 1');
  });

  it('should use frame name and field name if more than one frame', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          name: 'Series A',
          fields: [{ name: 'Field 1' }],
        }),
        toDataFrame({
          name: 'Series B',
          fields: [{ name: 'Field 1' }],
        }),
      ],
    });
    expect(title).toEqual('Series A Field 1');
  });

  it('should add field name count to name if it exists more than once and is equal to TIME_SERIES_VALUE_FIELD_NAME', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          fields: [{ name: TIME_SERIES_VALUE_FIELD_NAME }, { name: TIME_SERIES_VALUE_FIELD_NAME }],
        }),
      ],
    });
    const title2 = checkScenario({
      frames: [
        toDataFrame({
          fields: [{ name: TIME_SERIES_VALUE_FIELD_NAME }, { name: TIME_SERIES_VALUE_FIELD_NAME }],
        }),
      ],
      fieldIndex: 1,
    });

    expect(title).toEqual('Value 1');
    expect(title2).toEqual('Value 2');
  });

  it('should add field name count to name if field name exists more than once', () => {
    const title2 = checkScenario({
      frames: [
        toDataFrame({
          fields: [{ name: 'A' }, { name: 'A' }],
        }),
      ],
      fieldIndex: 1,
    });

    expect(title2).toEqual('A 2');
  });

  it('should only use label value if only one label', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          fields: [{ name: 'Value', labels: { server: 'Server A' } }],
        }),
      ],
    });
    expect(title).toEqual('Server A');
  });

  it('should use label value only if all series have same name', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          name: 'cpu',
          fields: [{ name: 'Value', labels: { server: 'Server A' } }],
        }),
        toDataFrame({
          name: 'cpu',
          fields: [{ name: 'Value', labels: { server: 'Server A' } }],
        }),
      ],
    });
    expect(title).toEqual('Server A');
  });

  it('should use label name and value if more than one label', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          fields: [{ name: 'Value', labels: { server: 'Server A', mode: 'B' } }],
        }),
      ],
    });
    expect(title).toEqual('{mode="B", server="Server A"}');
  });

  it('should use field name even when it is TIME_SERIES_VALUE_FIELD_NAME if there are no labels', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          fields: [{ name: TIME_SERIES_VALUE_FIELD_NAME, labels: {} }],
        }),
      ],
    });
    expect(title).toEqual('Value');
  });

  it('should use series name when field name is TIME_SERIES_VALUE_FIELD_NAME and there are no labels ', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          name: 'Series A',
          fields: [{ name: TIME_SERIES_VALUE_FIELD_NAME, labels: {} }],
        }),
      ],
    });
    expect(title).toEqual('Series A');
  });

  it('should reder loki frames', () => {
    const title = checkScenario({
      frames: [
        toDataFrame({
          refId: 'A',
          fields: [
            { name: 'time', type: FieldType.time },
            {
              name: 'line',
              labels: { host: 'ec2-13-53-116-156.eu-north-1.compute.amazonaws.com', region: 'eu-north1' },
            },
          ],
        }),
      ],
      fieldIndex: 1,
    });
    expect(title).toEqual('line {host="ec2-13-53-116-156.eu-north-1.compute.amazonaws.com", region="eu-north1"}');
  });
});
